from .rf_info import Frequency

__author__ = """Ian Perry"""
__email__ = 'ianperry99@gmail.com'
__version__ = '0.4.3'
__license__ = 'MIT'
__progname__ = 'rf-info'
__description__ = "Radio Frequency Information"
